/*
 Copyright (C)2011 Paul Houx
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
*/

#pragma once

#include "cinder/Camera.h"
#include "cinder/CinderMath.h"
#include "cinder/Vector.h"

namespace is
{
	using namespace ci;

	class CameraPerspAsym :
		public ci::CameraPersp
	{
	public:
		CameraPerspAsym();
		CameraPerspAsym( int pixelWidth, int pixelHeight, float fov ) 
			: CameraPersp(pixelWidth, pixelHeight, fov), mLensShiftX(0.0f), mLensShiftY(0.0f){};
		CameraPerspAsym( int pixelWidth, int pixelHeight, float fov, float nearPlane, float farPlane ) 
			: CameraPersp(pixelWidth, pixelHeight, fov, nearPlane, farPlane), mLensShiftX(0.0f), mLensShiftY(0.0f){};	

		virtual ~CameraPerspAsym();

		void	getLensShift(float *shiftX, float *shiftY){ *shiftX = mLensShiftX; *shiftY = mLensShiftY; };
		Vec2f	getLensShift(){ return Vec2f(mLensShiftX, mLensShiftY); };
		void	setLensShift(float shiftX, float shiftY);
		
		void	getNearClipCoordinates( Vec3f *topLeft, Vec3f *topRight, Vec3f *bottomLeft, Vec3f *bottomRight ) const;
		void	getFarClipCoordinates( Vec3f *topLeft, Vec3f *topRight, Vec3f *bottomLeft, Vec3f *bottomRight ) const;
	protected:
		void	calcProjection();
	protected:
		float	mLensShiftX;
		float	mLensShiftY;
	};
}